/*
    Copyright 2017-2018 Intel Corporation.

    The source code, information and material ("Material") contained herein is owned by
    Intel Corporation or its suppliers or licensors, and title to such Material remains
    with Intel Corporation or its suppliers or licensors. The Material contains
    proprietary information of Intel or its suppliers and licensors. The Material is
    protected by worldwide copyright laws and treaty provisions. No part of the Material
    may be used, copied, reproduced, modified, published, uploaded, posted, transmitted,
    distributed or disclosed in any way without Intel's prior express written permission.
    No license under any patent, copyright or other intellectual property rights in the
    Material is granted to or conferred upon you, either expressly, by implication,
    inducement, estoppel or otherwise. Any license under such intellectual property
    rights must be express and approved by Intel in writing.

    Unless otherwise agreed by Intel in writing, you may not remove or alter this notice
    or any other notice embedded in Materials by Intel or Intel's suppliers or licensors
    in any way.
*/

#ifndef __PSTL_bricks_impl_H
#define __PSTL_bricks_impl_H

namespace pstl {
namespace internal {

template<class _Iterator1, class _Iterator2, class _Function>
_Iterator2 brick_it_walk2(_Iterator1 __first1, _Iterator1 __last1, _Iterator2 __first2, _Function __f, /*vector=*/std::false_type) noexcept {
    for (; __first1 != __last1; ++__first1, ++__first2)
        __f(__first1, __first2);
    return __first2;
}

template<class _Iterator1, class _Iterator2, class _Function>
_Iterator2 brick_it_walk2(_Iterator1 __first1, _Iterator1 __last1, _Iterator2 __first2, _Function __f, /*vector=*/std::true_type) noexcept {
    return unseq_backend::simd_it_walk_2(__first1, __last1 - __first1, __first2, __f);
}

template<class _Iterator1, class _Size, class _Iterator2, class _Function>
_Iterator2 brick_it_walk2_n(_Iterator1 __first1, _Size __n, _Iterator2 __first2, _Function __f, /*vector=*/std::false_type) noexcept {
    for (; __n > 0; --__n, ++__first1, ++__first2)
        __f(__first1, __first2);
    return __first2;
}

template<class _Iterator1, class _Size, class _Iterator2, class _Function>
_Iterator2 brick_it_walk2_n(_Iterator1 __first1, _Size __n, _Iterator2 __first2, _Function __f, /*vector=*/std::true_type) noexcept {
    return unseq_backend::simd_it_walk_2(__first1, __n, __first2, __f);
}

} // namespace internal
} // namespace pstl

#endif /* __PSTL_bricks_impl_H */
