/*
    Copyright 2017-2018 Intel Corporation.

    The source code, information and material ("Material") contained herein is owned by
    Intel Corporation or its suppliers or licensors, and title to such Material remains
    with Intel Corporation or its suppliers or licensors. The Material contains
    proprietary information of Intel or its suppliers and licensors. The Material is
    protected by worldwide copyright laws and treaty provisions. No part of the Material
    may be used, copied, reproduced, modified, published, uploaded, posted, transmitted,
    distributed or disclosed in any way without Intel's prior express written permission.
    No license under any patent, copyright or other intellectual property rights in the
    Material is granted to or conferred upon you, either expressly, by implication,
    inducement, estoppel or otherwise. Any license under such intellectual property
    rights must be express and approved by Intel in writing.

    Unless otherwise agreed by Intel in writing, you may not remove or alter this notice
    or any other notice embedded in Materials by Intel or Intel's suppliers or licensors
    in any way.
*/

#ifndef __PSTL_glue_execution_defs_H
#define __PSTL_glue_execution_defs_H

#include <type_traits>

#include "execution_defs.h"

namespace std {
    // Type trait
    using pstl::execution::is_execution_policy;
#if __PSTL_CPP14_VARIABLE_TEMPLATES_PRESENT
#if __INTEL_COMPILER
    template<class T> constexpr bool is_execution_policy_v = is_execution_policy<T>::value;
#else
    using pstl::execution::is_execution_policy_v;
#endif
#endif

    namespace execution {
        // Standard C++ policy classes
        using pstl::execution::sequenced_policy;
#if __PSTL_USE_PAR_POLICIES
        using pstl::execution::parallel_policy;
        using pstl::execution::parallel_unsequenced_policy;
#endif
        // Standard predefined policy instances
        using pstl::execution::seq;
#if __PSTL_USE_PAR_POLICIES
        using pstl::execution::par;
        using pstl::execution::par_unseq;
#endif
        // Implementation-defined names
        // Unsequenced policy is not yet standard, but for consistency
        // we include it into namespace std::execution as well
        using pstl::execution::unseq;
        using pstl::execution::unsequenced_policy;
    } // namespace execution
} // namespace std

#endif /* __PSTL_glue_execution_defs_H */
