/*
    Copyright 2017-2018 Intel Corporation.

    The source code, information and material ("Material") contained herein is owned by
    Intel Corporation or its suppliers or licensors, and title to such Material remains
    with Intel Corporation or its suppliers or licensors. The Material contains
    proprietary information of Intel or its suppliers and licensors. The Material is
    protected by worldwide copyright laws and treaty provisions. No part of the Material
    may be used, copied, reproduced, modified, published, uploaded, posted, transmitted,
    distributed or disclosed in any way without Intel's prior express written permission.
    No license under any patent, copyright or other intellectual property rights in the
    Material is granted to or conferred upon you, either expressly, by implication,
    inducement, estoppel or otherwise. Any license under such intellectual property
    rights must be express and approved by Intel in writing.

    Unless otherwise agreed by Intel in writing, you may not remove or alter this notice
    or any other notice embedded in Materials by Intel or Intel's suppliers or licensors
    in any way.
*/

#ifndef __PSTL_memory_impl_H
#define __PSTL_memory_impl_H

#include <iterator>

#include "unseq_backend_simd.h"

namespace pstl {
namespace internal {

//------------------------------------------------------------------------
// uninitialized_move
//------------------------------------------------------------------------

template<class _ForwardIterator, class _OutputIterator>
_OutputIterator brick_uninitialized_move(_ForwardIterator __first, _ForwardIterator __last, _OutputIterator __result, /*vector=*/std::false_type) noexcept {
    typedef typename std::iterator_traits<_OutputIterator>::value_type _ValueType2;
    for (; __first != __last; ++__first, ++__result) {
        ::new (reduce_to_ptr(__result)) _ValueType2(std::move(*__first));
    }
    return __result;
}

template<class _ForwardIterator, class _OutputIterator>
_OutputIterator brick_uninitialized_move(_ForwardIterator __first, _ForwardIterator __last, _OutputIterator __result, /*vector=*/std::true_type) noexcept {
    typedef typename std::iterator_traits<_OutputIterator>::value_type __ValueType2;
    return unseq_backend::simd_it_walk_2(__first, __last - __first, __result,
        [](_ForwardIterator __first1, _OutputIterator first2) {::new (reduce_to_ptr(first2)) __ValueType2(std::move(*__first1));
    });
}

} // namespace internal
} // namespace pstl

#endif /* __PSTL_memory_impl_H */
